@@ -1,17 +1,17 @@
 ;;; borg.el --- Assimilate Emacs packages as Git submodules  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2016-2025 Jonas Bernoulli
+;; Copyright (C) 2016-2026 Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <emacs.borg@jonas.bernoulli.dev>
 ;; Homepage: https://github.com/emacscollective/borg
 ;; Keywords: tools
 
-;; Package-Version: 20251119.1634
-;; Package-Revision: 2669e518b176
+;; Package-Version: 20260101.1858
+;; Package-Revision: 61f9e083b8d9
 ;; Package-Requires: (
 ;;     (emacs "28.1")
 ;;     (epkg   "4.1")
-;;     (magit  "4.4"))
+;;     (magit  "4.5"))
 
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -46,6 +46,9 @@
 
 ;;; Code:
 
+(put 'if-let 'byte-obsolete-info nil)
+(put 'when-let 'byte-obsolete-info nil)
+
 (defun borg-report-load-duration (&optional file format-string)
   "Report how long it takes to load the file currently being loaded.
 
@@ -74,9 +77,10 @@ for the file name."
       (add-hook 'after-load-functions fn 98)
       (message format load-file))))
 
-(when (and (not after-init-time) user-init-file)
-  (borg-report-load-duration user-init-file nil))
-(borg-report-load-duration)
+(when user-init-file
+  (unless after-init-time
+    (borg-report-load-duration user-init-file nil))
+  (borg-report-load-duration))
 
 (defvar borg-debug-user-init-files t
   "Whether to extend the effect of `--debug-init' to user init files.
@@ -525,13 +529,13 @@ exists."
         (initialized 0))
     (borg-do-drones (drone)
       (cond
-       ((borg--drone-disabled-p drone)
-        (cl-incf skipped))
-       ((not (file-exists-p (borg-worktree drone)))
-        (cl-incf skipped))
-       (t
-        (cl-incf initialized)
-        (borg-activate drone))))
+        ((borg--drone-disabled-p drone)
+         (cl-incf skipped))
+        ((not (file-exists-p (borg-worktree drone)))
+         (cl-incf skipped))
+        (t
+         (cl-incf initialized)
+         (borg-activate drone))))
     (let* ((message (current-message))
            (inhibit (and message
                          (string-match-p
@@ -602,18 +606,18 @@ otherwise."
     (message "\n--- [%s] ---\n" drone)
     (let ((dir (borg-worktree drone)))
       (cond
-       ((borg--drone-disabled-p drone)
-        (message "Skipped (Disabled)"))
-       ((let ((min (cdr (assoc drone borg-minimal-emacs-alist))))
-          (and min (version< emacs-version min)
-               (message "Skipped (Requires Emacs >= %s)" min))))
-       ((not (file-exists-p dir))
-        (message "Skipped (Missing)"))
-       ((directory-empty-p dir)
-        (message "Skipped (Empty directory)"))
-       ((and quick (borg-get-all drone "build-step"))
-        (message "Skipped (Expensive to build)"))
-       ((borg-build drone nil native)))))
+        ((borg--drone-disabled-p drone)
+         (message "Skipped (Disabled)"))
+        ((let ((min (cdr (assoc drone borg-minimal-emacs-alist))))
+           (and min (version< emacs-version min)
+                (message "Skipped (Requires Emacs >= %s)" min))))
+        ((not (file-exists-p dir))
+         (message "Skipped (Missing)"))
+        ((directory-empty-p dir)
+         (message "Skipped (Empty directory)"))
+        ((and quick (borg-get-all drone "build-step"))
+         (message "Skipped (Expensive to build)"))
+        ((borg-build drone nil native)))))
   (borg-batch-rebuild-init))
 
 (defun borg-batch-rebuild-init ()
@@ -641,28 +645,28 @@ and optional NATIVE are both non-nil, then also compile natively."
     (setq clone (substring clone 0 -1)))
   (borg-clean clone)
   (cond
-   (noninteractive
-    (let ((borg-compile-function borg-compile-function))
-      (when (and (fboundp 'native-comp-available-p)
-                 (native-comp-available-p))
-        (when native
-          (setq borg-compile-function
-                (if (functionp native) native #'borg-byte+native-compile)))
-        (when (memq borg-compile-function '( borg--native-compile
-                                             native-compile
-                                             native-compile-async))
-          (message "WARNING: Using `%s' instead of unsuitable `%s'"
-                   'borg-byte+native-compile borg-compile-function)
-          (setq borg-compile-function #'borg-byte+native-compile)))
-      (borg--build-noninteractive clone)
-      (when activate
-        (borg-activate clone))))
-   ((let ((process (borg--build-interactive clone)))
-      (when activate
-        (add-function :after (process-sentinel process)
-                      (lambda (_process event)
-                        (when (string-equal event "finished\n")
-                          (borg-activate clone)))))))))
+    (noninteractive
+     (let ((borg-compile-function borg-compile-function))
+       (when (and (fboundp 'native-comp-available-p)
+                  (native-comp-available-p))
+         (when native
+           (setq borg-compile-function
+                 (if (functionp native) native #'borg-byte+native-compile)))
+         (when (memq borg-compile-function '( borg--native-compile
+                                              native-compile
+                                              native-compile-async))
+           (message "WARNING: Using `%s' instead of unsuitable `%s'"
+                    'borg-byte+native-compile borg-compile-function)
+           (setq borg-compile-function #'borg-byte+native-compile)))
+       (borg--build-noninteractive clone)
+       (when activate
+         (borg-activate clone))))
+    ((let ((process (borg--build-interactive clone)))
+       (when activate
+         (add-function :after (process-sentinel process)
+                       (lambda (_process event)
+                         (when (string-equal event "finished\n")
+                           (borg-activate clone)))))))))
 
 (defun borg--build-noninteractive (clone)
   (let ((default-directory (borg-worktree clone))
@@ -807,9 +811,9 @@ and optional NATIVE are both non-nil, then also compile natively."
                        (funcall fn string nil type))
                      '((name . "no-message")))
          ,bodyform)
-      (advice-remove 'progress-reporter-do-update #'ignore)
-      (advice-remove 'progress-reporter-done #'ignore)
-      (advice-remove 'byte-compile-info "no-message")))
+     (advice-remove 'progress-reporter-do-update #'ignore)
+     (advice-remove 'progress-reporter-done #'ignore)
+     (advice-remove 'byte-compile-info "no-message")))
 
 (defun borg-update-autoloads (clone &optional path)
   "Update autoload files for the clone named CLONE in the directories in PATH."
@@ -927,36 +931,36 @@ and optional NATIVE are both non-nil, then also compile natively."
 
 (defun borg-byte+native-compile (file)
   (cond
-   ((or (equal (getenv "NATIVE_DISABLED") "1")
-        (member (file-name-nondirectory file)
-                borg-native-compile-deny-list))
-    (byte-compile-file file))
-   ((and (fboundp 'comp--native-compile)
-         (fboundp 'comp-ensure-native-compiler))
-    (comp-ensure-native-compiler)
-    (let* ((byte+native-compile t)
-           (byte-to-native-output-buffer-file nil)
-           (native-compile-target-directory (car native-comp-eln-load-path))
-           (eln-file (comp--native-compile file)))
-      (pcase byte-to-native-output-buffer-file
-        (`(,temp-buffer . ,target-file)
-         (unwind-protect
-             (progn
-               (borg--byte-write-target-file temp-buffer target-file)
-               (when (stringp eln-file)
-                 (set-file-times eln-file)))
-           (kill-buffer temp-buffer)
-           (file-exists-p target-file))))))
-   ((error "Emacs %s does not support native compilation" emacs-version))))
+    ((or (equal (getenv "NATIVE_DISABLED") "1")
+         (member (file-name-nondirectory file)
+                 borg-native-compile-deny-list))
+     (byte-compile-file file))
+    ((and (fboundp 'comp--native-compile)
+          (fboundp 'comp-ensure-native-compiler))
+     (comp-ensure-native-compiler)
+     (let* ((byte+native-compile t)
+            (byte-to-native-output-buffer-file nil)
+            (native-compile-target-directory (car native-comp-eln-load-path))
+            (eln-file (comp--native-compile file)))
+       (pcase byte-to-native-output-buffer-file
+         (`(,temp-buffer . ,target-file)
+          (unwind-protect
+              (progn
+                (borg--byte-write-target-file temp-buffer target-file)
+                (when (stringp eln-file)
+                  (set-file-times eln-file)))
+            (kill-buffer temp-buffer)
+            (file-exists-p target-file))))))
+    ((error "Emacs %s does not support native compilation" emacs-version))))
 
 (defun borg-byte+native-compile-async (file)
   (byte-compile-file file)
   (cond
-   ((and (fboundp 'native-compile-async)
-         (fboundp 'comp-ensure-native-compiler))
-    (comp-ensure-native-compiler)
-    (native-compile-async file))
-   ((error "Emacs %s does not support native compilation" emacs-version))))
+    ((and (fboundp 'native-compile-async)
+          (fboundp 'comp-ensure-native-compiler))
+     (comp-ensure-native-compiler)
+     (native-compile-async file))
+    ((error "Emacs %s does not support native compilation" emacs-version))))
 
 (if (fboundp 'byte-write-target-file)
     (defalias 'borg--byte-write-target-file 'byte-write-target-file)
@@ -1095,8 +1099,8 @@ with a prefix argument, then also read the url in the minibuffer.
 With a negative prefix argument only add the submodule but don't
 build and activate the drone."
   (interactive
-   (nconc (borg-read-package "Assimilate package: " current-prefix-arg)
-          (list (< (prefix-numeric-value current-prefix-arg) 0))))
+    (nconc (borg-read-package "Assimilate package: " current-prefix-arg)
+           (list (< (prefix-numeric-value current-prefix-arg) 0))))
   (borg--maybe-confirm-unsafe-action "assimilate" package url)
   (message "Assimilating %s..." package)
   (let ((default-directory borg-top-level-directory))
@@ -1218,23 +1222,30 @@ Formatting is according to the commit message conventions."
      (process-lines "git" "diff-index" "--name-status" "--cached" "HEAD"
                     "--" (file-relative-name borg-drones-directory)))))
 
-(defun borg-report-after-init-duration ()
-  "Report how long it takes to run `after-init-hook'.
-This is done by adding two functions to that hook; one with a very
-low depth and the other with a very high depth.  Do nothing if the
-hook already ran."
+(defun borg-report-init-duration ()
+  "Report how long Emacs initialization and `after-init-hook' took.
+This adds functions to `after-init-hook' to determine how long it
+takes to run that, and to report those findings and also how long
+Emacs initialization took.  If that hook has already run, do nothing."
   (unless after-init-time
-    (letrec ((start nil)
-             (beg (lambda ()
-                    (setq start (current-time))
-                    (message "Running after-init-hook...")
-                    (remove-hook 'after-init-hook beg)))
-             (end (lambda ()
-                    (message "Running after-init-hook...done (%.3fs)"
-                             (float-time (time-subtract (current-time) start)))
-                    (remove-hook 'after-init-hook end))))
-      (add-hook 'after-init-hook beg -98)
-      (add-hook 'after-init-hook end +98))))
+    (letrec ((before nil)
+             (beg  (lambda ()
+                     (setq before (current-time))
+                     (message "Running after-init-hook...")
+                     (remove-hook 'after-init-hook beg)))
+             (end  (lambda ()
+                     (message "Running after-init-hook...done (%.3fs)"
+                              (float-time (time-subtract (current-time)
+                                                         before)))
+                     (remove-hook 'after-init-hook end)))
+             (init (lambda ()
+                     (message "Total init time %.3fs"
+                              (float-time (time-subtract after-init-time
+                                                         before-init-time)))
+                     (remove-hook 'after-init-hook init))))
+      (add-hook 'after-init-hook beg  -98)
+      (add-hook 'after-init-hook end  +98)
+      (add-hook 'after-init-hook init +98))))
 
 ;;; Integrations
 
@@ -1386,7 +1397,4 @@ Do you really want to %s it? "))))
 
 ;;; _
 (provide 'borg)
-;; Local Variables:
-;; indent-tabs-mode: nil
-;; End:
 ;;; borg.el ends here
